/*
    Kopete Oscar Protocol
    oscartypes.h - Oscar Type Definitions

    Copyright (c) 2004 Matt Rogers <matt.rogers@kdemail.net>

    Kopete (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#ifndef OSCARTYPES_H
#define OSCARTYPES_H

#include <qglobal.h>

namespace Yahoo
{
	typedef quint8 BYTE;
	typedef quint16 WORD;
	typedef quint32 DWORD;
}

#endif // OSCARTYPES_H
