<!DOCTYPE kpartgui>
<kpartgui version="13" name="kopete_wlm_chat">
	<MenuBar>
		<Menu noMerge="1" name="file">
			<text>&amp;Chat</text>
			<Action name="wlmSendNudge" />
		</Menu>
	</MenuBar>


	<ToolBar name="chatToolBar">
		<Action name="wlmSendNudge" />
		<Action name="wlmInvite" />
		<Action name="wlmSendInk" />
		<Action name="wlmSendVoice" />
	</ToolBar>
</kpartgui>
