/*
    transferbase.h - Base class of all I/O transfers
   
    Copyright (c) 2004      SUSE Linux AG	 	 http://www.suse.com

    Kopete (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>
 
    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#ifndef TRANSFERBASE_H
#define TRANSFERBASE_H

/**
@author Kopete Developers
*/
class TransferBase{
public:
    TransferBase();

    ~TransferBase();

};

#endif
